import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:spark_form/spark.dart';
import 'package:spark_form/spark_verification.dart';
import 'package:spark_form/spark_widgets.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  FormManage? _manageData;

  FormManage get _manage => _manageData!;

  final Map<String, String> _fieldMap = {
    'test': '1',
    'test1': '2',
    'test2': 't',
    'test3': 'sd',
    'test4': 'f',
  };

  final FormController _controller = FormController();

  void check() {
    _controller.validate();
  }

  void _handleSubmit(BuildContext context) {
    var result = _controller.validate(focusError: true);
    print(
        'check result: ${result.success} field: ${result.fieldName} error: ${result.errorMessage}');
    if(result.success){
      showModalBottomSheet(context: context, builder: (_) => const Center(
        child: Text('submit success'),
      ));
    }else{
      showModalBottomSheet(context: context, builder: (_) => const Center(
        child: Text('submit error', style: TextStyle(color: Colors.red),),
      ));
    }
    _manage.reset();
    print('reset form data');
  }

  @override
  void initState() {
    _manageData = FormManage.create(_fieldMap);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    double _labelWidth = 60.0;
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          backgroundColor: Colors.orangeAccent,
          title: const Text('Spark form demo'),
          elevation: 0,
        ),
        body: Column(
          children: [
            SingleChildScrollView(
              child: SparkForm(
                controller: _controller,
                manage: _manage,
                formBuilder: (BuildContext context) {
                  return Center(
                    child: Column(
                      children: [
                        SparkFormItem(
                          rules: [
                            SparkLengthVerification.length(
                              10,
                              errorMessage: 'length max',
                              type: VerifyType.change,
                            )
                          ],
                          spaceBetweenLayout: false,
                          label: 'input',
                          fieldName: 'test',
                          child: SizedBox(
                            width: 200,
                            child: SparkTextField(
                              manage: _manage['test'],
                              maxLines: 1,
                            ),
                          ),
                        ),
                        SparkFormItem(
                          readOnly: true,
                          label: 'select',
                          fieldName: 'test1',
                          child: SparkSelect(
                            manage: _manage['test1'],
                            options: [
                              SparkSelectOption.form('1', 'hahah'),
                              SparkSelectOption.form('2', 'hasdahah')
                            ],
                          ),
                        ),
                        SparkFormItem(
                          fieldName: 'test2',
                          label: 'title',
                          readOnly: true,
                          child: SparkLabelTitle(
                            manage: _manage['test2'],
                            onTap: (_updater) {
                              _updater(
                                  'test test test test test test test test test test test test test test');
                            },
                          ),
                        ),
                        Row(
                          children: [
                            SparkFormItem(
                              labelWidth: _labelWidth,
                              fieldName: 'test3',
                              label: 'title',
                              readOnly: true,
                              child: SparkLabelTitle(
                                manage: _manage['test3'],
                                onTap: (_updater) {
                                  _updater(
                                      'test test test test test test test test test test test test test test');
                                },
                                width: (MediaQuery.of(context).size.width -
                                        _labelWidth * 2 -
                                        92) /
                                    2,
                              ),
                            ),
                            SparkFormItem(
                              labelWidth: _labelWidth,
                              fieldName: 'test4',
                              label: 'title',
                              readOnly: true,
                              child: SparkLabelTitle(
                                manage: _manage['test4'],
                                onTap: (_updater) {
                                  _updater(
                                      'test test test test test test test test test test test test test test');
                                },
                                width: (MediaQuery.of(context).size.width -
                                        _labelWidth * 2 -
                                        92) /
                                    2,
                              ),
                            ),
                          ],
                        )
                      ],
                    ),
                  );
                },
              ),
            ),
            Builder(
                builder: (_) => GestureDetector(
                      onTap: () {
                        _handleSubmit(_);
                      },
                      child: Container(
                        margin: const EdgeInsets.only(top: 30),
                        width: MediaQuery.of(_).size.width - 40,
                        height: 50,
                        alignment: Alignment.center,
                        decoration: const BoxDecoration(
                          borderRadius: BorderRadius.all(Radius.circular(40)),
                          color: Colors.orangeAccent,
                        ),
                        child: const Text(
                          'Submit',
                          style: TextStyle(color: Colors.white, fontSize: 18.0),
                        ),
                      ),
                    ))
          ],
        ),
      ),
    );
  }
}
