part of spark;

class FormManage {
  FormManage._(Map<String, String?> createField) : _saveRawData = createField {
    _setupManage(createField);
  }

  /// create manage
  factory FormManage.create(Map<String, String?> createField) =>
      FormManage._(createField);

  /// manage save
  final Map<String, FormItemManage> _list = {};

  /// save init data
  final Map<String, String?> _saveRawData;

  /// get focus and controller info
  Map<String, FormItemManage> get manageList => _list;

  FormItemManage operator [](String index) {
    return _list[index]!;
  }

  /// setup field data
  void _setupManage(Map<String, String?> createField) {
    createField.forEach((key, value) {
      _list[key] = FormItemManage.create(init: value ?? '');
    });
  }

  /// reset manage by default data
  void reset() {
    for(MapEntry<String, String?> _entry in _saveRawData.entries){
      _list[_entry.key]!.controller.text = _entry.value ?? "";
    }
  }

  /// clear manage data
  void clear(){
    for(MapEntry<String, String?> _entry in _saveRawData.entries){
      _list[_entry.key]!.controller.text = "";
    }
  }

  /// get json data
  /// exclude field[excludeField]is not include in json data
  Map getJson({List<String>? excludeField}) {
    Map<String, dynamic> _jsonData = {};
    _list.forEach((key, value) {
      if (excludeField == null || !excludeField.contains(key)) {
        _jsonData[key] = value.controller.text;
      }
    });
    return _jsonData;
  }

  /// dispose all
  void dispose() {
    for (var _recordItem in _list.values) {
      _recordItem.dispose();
    }
  }
}

class FormItemManage {
  /// create manage
  FormItemManage.create({String? init}) {
    _controller = TextEditingController(text: init);
    _focusNode = FocusNode();
  }

  TextEditingController? _controller;
  FocusNode? _focusNode;
  VoidCallback? clickCallback;

  TextEditingController get controller => _controller!;

  FocusNode get focusNode => _focusNode!;

  /// regeste click callback
  /// will call on form click
  /// see [FocusPointerScope]
  void registerClickCallback(VoidCallback callback) {
    clickCallback = callback;
  }

  /// clean callback
  void clearClickCallback() {
    clickCallback = null;
  }

  /// dispose all
  void dispose() {
    if (_controller != null) {
      TextEditingController _temp = _controller!;
      _controller = null;
      _temp.dispose();
    }
    if (_focusNode != null) {
      FocusNode _temp = _focusNode!;
      _focusNode = null;
      _temp.dispose();
    }
    clickCallback = null;
  }
}

class FormDataChangeNotification extends Notification {
  /// form data change notification
  FormDataChangeNotification({
    this.changeField,
    required this.oldValue,
    required this.newValue,
  });

  /// form field
  String? changeField;

  /// form item widget old value
  dynamic oldValue;

  /// form item widget new value
  dynamic newValue;
}

class SparkFormData extends InheritedWidget {
  const SparkFormData({
    required this.formData,
    Key? key,
    required Widget child,
  }) : super(key: key, child: child);

  final SparkFormItemData formData;

  /// get [SparkFormItemData] of context
  static SparkFormItemData? of(BuildContext context) {
    return context
        .dependOnInheritedWidgetOfExactType<SparkFormData>()
        ?.formData;
  }

  @override
  bool updateShouldNotify(covariant InheritedWidget oldWidget) {
    return true;
  }
}

class SparkFormItemData {
  Map<String, ValidateCallback> validateList = <String, ValidateCallback>{};

  FormManage? _manage;

  FormManage get manage => _manage!;

  void _setManage(FormManage manage) {
    _manage = manage;
  }

  /// when bind widget dispose call it's
  void disposeField(String field) {
    validateList.remove(field);
  }

  /// clear all bind
  void dispose() {
    validateList.clear();
  }
}

class FormController {
  SparkFormItemData? _data;

  FormManage? _manage;

  FormManage get manage => _manage!;

  SparkFormItemData get data => _data!;

  /// use in form init, set controller data
  void _setFormItemData(SparkFormItemData data, FormManage manageData) {
    _data = data;
    _manage = manageData;
  }

  ValidateResult validateField(String fieldName, VerifyType type) {
    ValidateResult Function(dynamic, VerifyType)? _fieldCheck =
        data.validateList[fieldName];
    FormItemManage _fieldData = manage[fieldName];
    assert(_fieldCheck != null,
        'get null check rule, please check form item fieldName');
    return _fieldCheck!(_fieldData.controller.text, type);
  }

  /// validate all rules, get validate result
  /// [focusError] auto focus first error
  ValidateResult validate({bool? focusError}) {
    Map<String, ValidateResult Function(dynamic, VerifyType)> _list =
        data.validateList;
    for (MapEntry<String, ValidateResult Function(dynamic, VerifyType)> _item
        in _list.entries) {
      FormItemManage _fieldData = manage[_item.key];
      ValidateResult _result = _item.value(
        _fieldData.controller.text,
        VerifyType.all,
      );
      if (!_result.success) {
        if (focusError == true) {
          _fieldData.focusNode.requestFocus();
        }
        return _result;
      }
    }
    return ValidateResult.from(success: true);
  }

  /// reset all field data
  void resetAll(){
    _manage?.reset();
  }

  /// clean all data
  void cleanAll(){
    _manage?.clear();
  }
}

abstract class SparkOption {
  SparkOption(this._label, this._value);

  final String? _label;

  final String? _value;

  String get label => _label!;

  String get value => _value!;
}
