part of spark;

/// default primary color
const Color _primaryDefaultColor = Color(0xFF333333);

/// lable width
const double _defaultLabelWidth = 60;

/// border color
const Color _defaultBorderColor = Color(0xFF779AFD);

/// hit text color
const Color _hitTextColor = Color(0xFFF5F5F5);

/// error color
const Color _errorOrRequiredColor = Color(0xFFe74c3c);

/// default font size
const double _defaultFormItemFontSize = 14.0;

/// default form color
const Color _defaultFormItemColor = Color(0xFF333333);

/// all form item disabled color
const Color _defaultFormItemDisabledColor = Color(0xFF999999);

/// select option min size
const Size _defaultSelectItemSize = Size(76, 30);

/// spark select option selected style
const SparkSelectDecoration _defaultSelectedDecoration = SparkSelectDecoration(
  fontSize: 13.0,
  textColor: Color(0xFF2878FF),
  backgroundColor: Color(0x1A1677FF),
  border: Border.fromBorderSide(BorderSide(color: Color(0xff2878FF), width: 1)),
  borderRadius: BorderRadius.all(Radius.circular(20)),
);

/// spark select option unSelected style
const SparkSelectDecoration _defaultUnSelectedDecoration =
    SparkSelectDecoration(
  fontSize: 13.0,
  textColor: Color(0xff646474),
  backgroundColor: Color(0xffF6F8FA),
  border: Border.fromBorderSide(BorderSide(color: Color(0x1A1677FF), width: 1)),
  borderRadius: BorderRadius.all(Radius.circular(20)),
);

/// form item content box decoration
const FocusDecoration _defaultFormDecoration = FocusDecoration(
  focusBorder: Border(bottom: BorderSide(color: _defaultBorderColor, width: 1)),
  unFocusBorder: Border(bottom: BorderSide(color: Color(0x1A1677FF), width: 1)),
);

/// form item content padding
const EdgeInsets _defaultFormItemPadding =
    EdgeInsets.symmetric(horizontal: 14, vertical: 14);

/// form item content margin
const EdgeInsets _defaultFormItemMargin = EdgeInsets.zero;

/// default font size
const double _baseFontSize = 14.0;

class SparkThemeData extends InheritedWidget {
  /// form theme data manage provider
  const SparkThemeData({required Widget child, required this.theme, Key? key})
      : super(child: child, key: key);

  /// form theme
  final SparkTheme theme;

  /// get form theme data of context
  static SparkTheme? of(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<SparkThemeData>()?.theme;
  }

  @override
  bool updateShouldNotify(covariant InheritedWidget oldWidget) {
    return true;
  }
}

class SparkTheme {
  /// spark form theme
  /// default use in [SparkForm]
  const SparkTheme({
    this.labelColor = _primaryDefaultColor,
    this.hitColor = _hitTextColor,
    this.labelFontSize = _baseFontSize,
    this.labelFontWeight,
    this.requiredColor = _errorOrRequiredColor,
    this.requiredPrefix = '*',
    this.formItemFontSize = _defaultFormItemFontSize,
    this.formItemColor = _defaultFormItemColor,
    this.formItemDisabledColor = _defaultFormItemDisabledColor,
    this.lableWidth = _defaultLabelWidth,
    this.lablePadding = EdgeInsets.zero,
    this.formItemDecoration = _defaultFormDecoration,
    this.formItemPadding = _defaultFormItemPadding,
    this.formItemMargin = _defaultFormItemMargin,
    this.selectItemSize = _defaultSelectItemSize,
    this.selectedItemDecoration = _defaultSelectedDecoration,
    this.unSelectedItemDecoration = _defaultUnSelectedDecoration,
  });

  final double? lableWidth;

  final String? requiredPrefix;

  final Color? requiredColor;

  final Color? labelColor;

  final double? labelFontSize;

  final FontWeight? labelFontWeight;

  final Color? hitColor;

  final double? formItemFontSize;

  final Color? formItemColor;

  final Color? formItemDisabledColor;

  final EdgeInsets? lablePadding;

  final FocusDecoration? formItemDecoration;

  final EdgeInsets? formItemPadding;

  final EdgeInsets? formItemMargin;

  final Size? selectItemSize;

  final SparkSelectDecoration? selectedItemDecoration;

  final SparkSelectDecoration? unSelectedItemDecoration;
}
