import 'package:spark_form/spark.dart';

enum VerifyType {
  /// verfiy value after changed
  change,

  /// verfiy value after blur
  blur,

  /// include change and blur
  all,
}

abstract class SparkVerification {
  SparkVerification({
    required this.errorMessage,
    required this.type,
    this.strategy,
  });

  /// verify error message
  String errorMessage;

  /// validate type
  VerifyType? type;

  /// validate strategy
  ///
  FormStrategy? strategy;

  /// validate rule
  ValidateResult validate(dynamic value, String fieldName);
}

class SparkLengthVerification extends SparkVerification {
  int? length;

  SparkLengthVerification.length(
    this.length, {
    required String errorMessage,
    required VerifyType? type,
    FormStrategy? strategy = FormStrategy.limit,
  }) : super(
          errorMessage: errorMessage,
          type: type,
          strategy: strategy,
        );

  @override
  ValidateResult validate(dynamic value, String fieldName) {
    return ValidateResult.from(
      errorMessage: errorMessage,
      fieldName: fieldName,
      success: value.length <= length,
      strategy: strategy,
    );
  }
}

class SparkFormatVerification extends SparkVerification {
  RegExp? rule;

  /// valify result by regexp
  SparkFormatVerification.form(
    this.rule, {
    required String errorMessage,
    required VerifyType? type,
    FormStrategy? strategy = FormStrategy.limit,
  }) : super(
          errorMessage: errorMessage,
          type: type,
          strategy: strategy,
        );

  @override
  ValidateResult validate(dynamic value, String fieldName) {
    return ValidateResult.from(
        errorMessage: errorMessage,
        fieldName: fieldName,
        success: rule!.hasMatch(value),
        strategy: strategy);
  }
}

class ValidateResult {
  ValidateResult.from({
    this.errorMessage,
    this.fieldName,
    required this.success,
    this.strategy,
  });

  /// result tip message
  String? errorMessage;

  /// result
  bool success;

  /// check field
  String? fieldName;

  /// use strategy type
  FormStrategy? strategy;
}
