part of spark_widget;

class SparkLabelTitle extends StatefulWidget {
  const SparkLabelTitle({
    Key? key,
    required this.manage,
    this.suffixIcon,
    this.showSuffix,
    this.suffixSize,
    this.maxLines,
    this.onTap,
    this.hitText,
    this.style,
    this.hitTextStyle,
    this.padding,
    this.margin,
    this.width,
  }) : super(key: key);

  final FormItemManage manage;

  final Widget? suffixIcon;

  final bool? showSuffix;

  final Size? suffixSize;

  final int? maxLines;

  final Function(ValueUpdater updater)? onTap;

  final String? hitText;

  final TextStyle? style;

  final TextStyle? hitTextStyle;

  final EdgeInsets? padding;

  final EdgeInsets? margin;

  final double? width;

  @override
  State<StatefulWidget> createState() => _SparkLabelTitleBase();
}

class _SparkLabelTitleBase extends State<SparkLabelTitle> {
  FormItemManage get manage => widget.manage;

  void _clickHandle() {
    if (widget.onTap != null) {
      widget.onTap!((value) {
        manage.controller.text = value;
      });
    }
  }

  @override
  initState() {
    manage.registerClickCallback(_clickHandle);
    super.initState();
  }

  Widget _buildLabelTitle() {
    String _fieldValue = manage.controller.text;
    bool isHitText = false;
    if (emptyOrNullString(_fieldValue)) {
      _fieldValue = widget.hitText ?? '';
      isHitText = true;
    }
    TextStyle? _style = isHitText ? widget.style : widget.hitTextStyle;
    return Container(
      alignment: Alignment.topRight,
      padding: widget.padding,
      margin: widget.margin,
      width: widget.width ?? 200,
      child: Text(
        _fieldValue,
        style: _style,
        maxLines: widget.maxLines,
        overflow: TextOverflow.ellipsis,
      ),
    );
  }

  Widget? _buildSuffix() {
    if (widget.showSuffix ?? false) {
      return null;
    }
    Size _size = widget.suffixSize ?? const Size(18, 18);
    return SizedBox(
      width: _size.width,
      height: _size.height,
      child: widget.suffixIcon ?? _buildDefaultSuffixIcon(),
    );
  }

  Widget _buildDefaultSuffixIcon() {
    return const Icon(
      Icons.keyboard_arrow_right_rounded,
      size: 20,
    );
  }

  @override
  dispose() {
    // just remove
    manage.clearClickCallback();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    Widget? _suffix = _buildSuffix();
    return SparkWrapper(
      manage: manage,
      builder: (_, _value, _updater) {
        return Row(
          children: [
            _buildLabelTitle(),
            if (_suffix != null) _suffix,
          ],
        );
      },
    );
  }
}
