part of spark_widget;

class SparkTextField extends SparkFormWidget {
  const SparkTextField({
    Key? key,
    required FormItemManage manage,
    this.initialValue,
    this.keyboardType,
    this.textInputAction,
    this.style,
    this.strutStyle,
    this.textDirection,
    this.textAlignVertical,
    this.toolbarOptions,
    this.showCursor,
    this.smartDashesType,
    this.smartQuotesType,
    this.maxLengthEnforcement,
    this.minLines,
    this.maxLength,
    this.onChanged,
    this.onTap,
    this.onEditingComplete,
    this.onFieldSubmitted,
    this.onSaved,
    this.validator,
    this.inputFormatters,
    this.enabled,
    this.cursorHeight,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.selectionControls,
    this.buildCounter,
    this.scrollPhysics,
    this.autofillHints,
    this.autovalidateMode,
    this.scrollController,
    this.decoration,
    this.textCapitalization,
    this.autofocus,
    this.readOnly,
    this.obscuringCharacter,
    this.obscureText,
    this.autocorrect,
    this.enableSuggestions,
    this.maxLines,
    this.expands,
    this.cursorWidth,
    this.scrollPadding,
    this.enableIMEPersonalizedLearning,
    this.enableInteractiveSelection,
  }) : super(key: key, manage: manage);

  final String? initialValue;
  final InputDecoration? decoration;
  final TextInputType? keyboardType;
  final TextCapitalization? textCapitalization;
  final TextInputAction? textInputAction;
  final TextStyle? style;
  final StrutStyle? strutStyle;
  final TextDirection? textDirection;
  final TextAlign textAlign = TextAlign.start;
  final TextAlignVertical? textAlignVertical;
  final bool? autofocus;
  final bool? readOnly;
  final ToolbarOptions? toolbarOptions;
  final bool? showCursor;
  final String? obscuringCharacter;
  final bool? obscureText;
  final bool? autocorrect;
  final SmartDashesType? smartDashesType;
  final SmartQuotesType? smartQuotesType;
  final bool? enableSuggestions;
  final MaxLengthEnforcement? maxLengthEnforcement;
  final int? maxLines;
  final int? minLines;
  final bool? expands;
  final int? maxLength;
  final ValueChanged<String>? onChanged;
  final GestureTapCallback? onTap;
  final VoidCallback? onEditingComplete;
  final ValueChanged<String>? onFieldSubmitted;
  final FormFieldSetter<String>? onSaved;
  final FormFieldValidator<String>? validator;
  final List<TextInputFormatter>? inputFormatters;
  final bool? enabled;
  final double? cursorWidth;
  final double? cursorHeight;
  final Radius? cursorRadius;
  final Color? cursorColor;
  final Brightness? keyboardAppearance;
  final EdgeInsets? scrollPadding;
  final TextSelectionControls? selectionControls;
  final InputCounterWidgetBuilder? buildCounter;
  final ScrollPhysics? scrollPhysics;
  final Iterable<String>? autofillHints;
  final AutovalidateMode? autovalidateMode;
  final ScrollController? scrollController;
  final bool? enableIMEPersonalizedLearning;
  final bool? enableInteractiveSelection;

  @override
  State<StatefulWidget> createState() => _SparkTextFieldBase();
}

class _SparkTextFieldBase extends SparkFormWidgetBase<SparkTextField> {
  @override
  void initState() {
    oldValue = '';
    super.initState();
  }

  /// handle textField input change
  /// notify parent form item handle value change
  /// see [SparkFormItem]
  void _handleChange(String value) {
    handleFormDataChange(value, oldValue);
    oldValue = widget.manage.controller.text;
  }

  @override
  Widget build(BuildContext context) {
    SparkTheme? themeData = SparkThemeData.of(context);
    bool obscureText = widget.obscureText ?? false;

    bool _enable = widget.enabled ?? widget.decoration?.enabled ?? true;
    return TextField(
      controller: widget.manage.controller,
      focusNode: widget.manage.focusNode,
      decoration: widget.decoration,
      keyboardType: widget.keyboardType,
      textInputAction: widget.textInputAction,
      style: widget.style ??
          TextStyle(
            color: _enable
                ? themeData?.formItemColor
                : themeData?.formItemDisabledColor,
            fontSize: themeData?.formItemFontSize,
          ),
      strutStyle: widget.strutStyle,
      textAlign: widget.textAlign,
      textAlignVertical: widget.textAlignVertical,
      textDirection: widget.textDirection,
      textCapitalization: widget.textCapitalization ?? TextCapitalization.none,
      autofocus: widget.autofocus ?? false,
      toolbarOptions: widget.toolbarOptions,
      readOnly: widget.readOnly ?? false,
      showCursor: widget.showCursor,
      obscuringCharacter: widget.obscuringCharacter ?? '•',
      obscureText: obscureText,
      autocorrect: widget.autocorrect ?? true,
      smartDashesType: widget.smartDashesType ??
          (obscureText ? SmartDashesType.disabled : SmartDashesType.enabled),
      smartQuotesType: widget.smartQuotesType ??
          (obscureText ? SmartQuotesType.disabled : SmartQuotesType.enabled),
      enableSuggestions: widget.enableSuggestions ?? true,
      maxLengthEnforcement: widget.maxLengthEnforcement,
      maxLines: widget.maxLines,
      minLines: widget.minLines,
      expands: widget.expands ?? false,
      maxLength: widget.maxLength,
      onChanged: _handleChange,
      onTap: widget.onTap,
      onEditingComplete: widget.onEditingComplete,
      onSubmitted: widget.onFieldSubmitted,
      inputFormatters: widget.inputFormatters,
      enabled: _enable,
      cursorWidth: widget.cursorWidth ?? 2.0,
      cursorHeight: widget.cursorHeight,
      cursorRadius: widget.cursorRadius,
      cursorColor: widget.cursorColor,
      scrollPadding: widget.scrollPadding ?? const EdgeInsets.all(20.0),
      scrollPhysics: widget.scrollPhysics,
      keyboardAppearance: widget.keyboardAppearance,
      enableInteractiveSelection: widget.enableInteractiveSelection ?? true,
      selectionControls: widget.selectionControls,
      buildCounter: widget.buildCounter,
      autofillHints: widget.autofillHints,
      scrollController: widget.scrollController,
      enableIMEPersonalizedLearning:
          widget.enableIMEPersonalizedLearning ?? true,
    );
  }
}
